@@ -14,6 +14,7 @@ it can be extended in a simple way by just inheriting from CommandV.
 import logging
 import os
 import re
+import requests_unixsocket
 import yaml
 import simplejson as json
 import base64
@@ -44,11 +45,17 @@ STATES = [
     State('Creating container'),
     State('Deactivating database'),
     State('Restoring backup'),
+    State('Restoring btrfs backup'),
+    State('Restoring zfs backup'),
     State('Generating backup'),
+    State('Generating btrfs backup'),
+    State('Generating zfs backup'),
     State('Deactivating backup'),
     State('Changing passwords'),
     State('Updating database'),
     State('Destroying instance'),
+    State('Destroying btrfs instance'),
+    State('Destroying zfs instance'),
     State('Pushing images'),
     State('Updating production instance'),
     State('Restarting container'),
@@ -199,6 +206,18 @@ class CommandV(Extensible):
         """
         return self.__config.reloading_config
 
+    def use_btrfs(self):
+        """Transtions condition used to know how to handle the databases (delete, restore, backup)
+        """
+        instance_type = self.__config.instance_config.get('instance_type')
+        return self.__config.backup_type == 'btrfs' and instance_type != 'production'
+
+    def use_zfs(self):
+        """Transtions condition used to know how to handle the databases (delete, restore, backup)
+        """
+        instance_type = self.__config.instance_config.get('instance_type')
+        return self.__config.backup_type == 'zfs' and instance_type != 'production'
+
     def _can_add_new(self):
         """As we have a limited amount of resources in each node we need to check if the running
         instances count is less than the allowed to run a new one.
@@ -244,17 +263,28 @@ class CommandV(Extensible):
         self.sender.send_message('Image {} built.'.format(
             res.get('result').get('image_name')))
         base_img = self.__config.container_config.get('customer_image')
-        tag_img = '{img}:{tag}'.format(
-            img=base_img, tag=res['result'].get('image_sha'))
+        if self.instance_manager.config_variables.get('docker_image_repo'):
+            base_img = self.instance_manager.config_variables.get('docker_image_repo')
+            tag_img = '{app}-{version}-{sha}'.format(
+                app=self.instance_manager.config_variables.get('main_app'),
+                version=self.instance_manager.config_variables.get('version'),
+                sha=res['result'].get('image_sha'))
+        else:
+            logger.warning('*** *** *** WARNING *** *** ***')
+            logger.warning('You are using a legacy mode for tag docker images.'
+                           'New mode requires set docker_image_repo environment variable.')
+            logger.warning('*** *** *** *** *** *** *** ***')
+            tag_img = res['result'].get('image_sha')
+        full_image_name = '{img}:{tag}'.format(img=base_img, tag=tag_img)
         if not isinstance(self.__config.container_config.get('push_image'), list):
             self.__config.container_config.update({'push_image': []})
         instance_type = self.__config.instance_config.get('instance_type')
         self.__config.container_config.get('push_image').append({
-            'tag': res['result'].get('image_sha'), 'base': base_img,
+            'tag': tag_img, 'base': base_img,
             'image': res.get('result').get('image_name'),
             'tag_latest': bool(instance_type == 'updates')})
         self.__config.container_config.update({
-            'image_name': (tag_img if not self.skip_push() else
+            'image_name': (full_image_name if not self.skip_push() else
                            res.get('result').get('image_name'))
         })
         self.build_done_trigger()
@@ -515,6 +545,96 @@ class CommandV(Extensible):
         self.restore_done_trigger()
         return res
 
+    def _make_socket_request(self, driver, endpoint, method):
+        """Performs a request to a DeployV socket that can handle btrfs/zfs backups.
+
+        :param driver: Driver that should handle the requests, this can be `btrfs` or `zfs`.
+        :param endpoint: Controller endpoint for the request, this can be `restore` or `backup`.
+        :param method: Method that will be used for the request to the endpoint,
+                       e.g. post, delete, get, etc.
+
+        :return: Request object with the result of the request.
+        """
+        cfg = self.__config
+        data = {
+            "customer_id": cfg.instance_config.get("customer_id"),
+            "task_id": cfg.instance_config.get("task_id"),
+            "psql_version": self.instance_manager.get_postgres_version(),
+            "db_name": cfg.instance_config.get('config').get('db_name'),
+            "db_user": cfg.instance_config.get("config").get("db_user"),
+            "current_db_name": cfg.current_db_name,
+            "current_db_user": cfg.current_db_user,
+            "backup_type": cfg.backup_type,
+            "backup_path": cfg.backup_src or os.getcwd(),
+            "db_password": cfg.instance_config.get("config").get("db_password"),
+            "instance_name": cfg.instance_config.get("config").get("container_name"),
+            "driver": driver
+        }
+        headers = {"Content-Type": "application/json"}
+        url = ("http+unix://%2Fvar%2Frun%2Fdeployv%2Fdeployv.sock/{endpoint}"
+               .format(endpoint=endpoint))
+        session = requests_unixsocket.Session()
+        if not hasattr(session, method):
+            logger.error('The %s object has no attribute %s', type(session), method)
+        func = getattr(session, method)
+        res = func(url, headers=headers, json=data)
+        return res
+
+    @events
+    def restore_snapshot(self):
+        """Creates a new database based on a snapshot taken with btrfs or zfs.
+        """
+        self.sender.send_message('Creating database')
+        self.__config.check_config('restore')
+        res = {'command': 'restore', 'result': {}}
+        database_name = self.__config.instance_config.get('config').get('db_name')
+        self.__config.instance_config.get('config').update({'db_port': 5432})
+        self.sender.send_message('Restoring a backup in the database "{}".'
+                                 .format(database_name))
+        request_res = self._make_socket_request(self.__config.backup_type,
+                                                'restore', 'post').json()
+        if request_res.get('code') and request_res.get('code') != 200:
+            res.update(request_res)
+            self.send_message_error(res)
+            return res
+        res.get('result').update(request_res)
+        logger.debug('restore_res: %s', str(res))
+        # The zfs socket does not return the abs path of the filestore.
+        if not os.path.isabs(res['result'].get('filestore_path')):
+            res['result'].update({'filestore_path': '/' + res['result'].get('filestore_path')})
+        volumes = {
+            res['result'].get('filestore_path'): {
+                'bind': self.__config.container_config.get('volumes').get('filestore'),
+                'ro': False
+                }
+        }
+        postgres_container = res['result'].get('container')
+        postgres_info = self.instance_manager.inspect(postgres_container)
+        db_host = postgres_info.get('NetworkSettings').get('Gateway')
+        db_port = res['result'].get('port')
+        self.instance_manager.config.get('env_vars').update({
+            'db_host': db_host, 'db_port': db_port, 'external_db_port': db_port})
+        self.instance_manager._InstanceV__full_config.instance_config.get('config').update({
+            'db_host': db_host, 'db_port': db_port})
+        self.__config.container_config.get('volumes').update({
+            'filestore': '{fs_path}:{vol}'.format(
+                fs_path=res['result'].get('filestore_path'),
+                vol=self.__config.container_config.get('volumes').get('filestore'))})
+        db_config = self.__config.instance_config.get('config').copy()
+        db_config.update({'db_port': db_port, 'db_host': db_host})
+        user = self.instance_manager.get_odoo_users(database_name,
+                                                    user_id=self.get_admin_id())
+        res.get('result').update({'user_admin': (user and user[0] or {}).get("login")})
+        params = {'database.generated_at': res["result"].get('create_date'),
+                  'database.type': self.instance_manager.instance_type}
+        self.instance_manager.set_parameters(database_name, params)
+        self.new_deactivate(db_config)
+        bash_lines = ["chown -R odoo:odoo /home/odoo/.local"]
+        self.instance_manager.execute_in_auxiliary_container(database_name, bash_lines, volumes)
+        self.sender.send_message(msg='Database "{}" created.'.format(database_name), body=res)
+        self.restore_done_trigger()
+        return res
+
     def _attach_file(self, msg, file_name, file_type):
         res = msg
         if not res.get('attachments', False):
@@ -761,7 +881,9 @@ class CommandV(Extensible):
             tag = match.groupdict().get('tag')
             instance_type = self.__config.instance_config.get('instance_type')
             self.__config.container_config.get('push_image').append({
-                'tag': tag, 'base': self.__config.container_config.get('customer_image'),
+                'tag': tag,
+                'base': self.instance_manager.config_variables.get('docker_image_repo')
+                or self.__config.container_config.get('customer_image'),
                 'tag_latest': bool(instance_type == 'updates')})
         if self.__config.instance_config.get('instance_type') == 'updates':
             self.get_db_data(db_name, 'post-update')
@@ -840,8 +962,7 @@ class CommandV(Extensible):
                 remove_img = self.instance_manager.remove_image(config.get('image_name'))
                 result += remove_img
             if not config.get('keep_db'):
-                db_name = self.__config.instance_config.get('config').get('db_name')
-                res_dropdb = self._drop_dbs([db_name])
+                res_dropdb = self.delete_database()
                 result.extend(res_dropdb)
             res.update({
                 'result': result
@@ -862,6 +983,22 @@ class CommandV(Extensible):
         self.destroy_done_trigger()
         return res
 
+    def delete_database(self):
+        """Method used to check if the database should be deleted by this worker
+        using `DROP DATABASE` or if it should be requested to a btrfs/zfs driver.
+        """
+        db_name = self.__config.instance_config.get('config').get('db_name')
+        if not self.use_btrfs() and not self.use_zfs():
+            res_dropdb = self._drop_dbs([db_name])
+            return res_dropdb
+        request_res = self._make_socket_request(self.__config.backup_type,
+                                                'restore', 'delete').json()
+        if request_res.get('code') and request_res.get('code') != 200:
+            raise errors.CommandError(
+                'delete', 'Failed to remove the database: %s' % request_res.get('error'))
+        res = ['Dropped %s database' % db_name]
+        return res
+
     def get_containers(self):
         """Gets the containers list that are currently executing, which names match the
         regex: '^/([tiu][0-9]+_[a-z]+[0-9]+)_odoo', for example: 't111_cust80_odoo'.
@@ -1129,14 +1266,12 @@ class CommandV(Extensible):
             'update_logs': start_result.get('result').get('update_logs'),
             'result_type': 'pre-backup'
         })
-        odoo_user = self.__config.container_config.get('env_vars').get('odoo_user')
-        odoo_home = os.path.join(self.__config.container_config.get('env_vars').get('odoo_home'),
-                                 'instance')
-        path_branches = os.path.join(self.instance_manager.temp_folder, 'repositories.json')
+        odoo_home = self.__config.container_config.get('env_vars').get('odoo_home')
         self.__instance_manager.exec_cmd(
-            'su {user} -c "branchesv save -p {home} -f /tmp/repositories.json"'
-            .format(user=odoo_user, home=odoo_home)
+            'bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"'
+            .format(home=odoo_home)
         )
+        path_branches = os.path.join(self.instance_manager.temp_folder, 'repositories.json')
         start_result.get('attachments').append({
             'file_name': os.path.basename(path_branches),
             'file': utils.generate_attachment(path_branches),
@@ -1180,14 +1315,17 @@ class CommandV(Extensible):
         image_name = self.__config.container_config.get('image_name')
         image_repo = image_name.split(':')[0]
         image_tag = utils.get_strtime()
-        self.sender.send_message('Pushing image {}:{}'.format(image_name, image_tag))
+        self.sender.send_message('Pushing image {}:{}'.format(image_repo, image_tag))
         commit_result = manager.commit_and_push(image_repo, image_tag)
         if commit_result.get('error'):
             res.update({'error': commit_result.get('error')})
             return res
         # Tag updates as stable and push
         self.sender.send_message('Pushing scheduled image {} as stable'.format(image_name))
-        tag_result = manager.tag_and_push(image_name, image_repo, tag='stable')
+        variables = self.instance_manager.config_variables
+        tag = "%(app)s-%(version)s-stable" % {
+            "app": variables.get("main_app"), "version": variables.get("version")}
+        tag_result = manager.tag_and_push(image_name, image_repo, tag=tag)
         if tag_result.get('error'):
             res.update({'error': tag_result.get('error')})
         res['result'].update({
@@ -1312,11 +1450,10 @@ class CommandV(Extensible):
         res = {'result': False}
         manager = self.instance_manager
         odoo_home = self.__config.container_config.get('env_vars').get('odoo_home')
-        odoo_user = self.__config.container_config.get('env_vars').get('odoo_user')
         # Get the info of the repos
         cmd_output = manager.exec_cmd(
-            ('su {user} -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
-            .format(user=odoo_user, home=odoo_home)
+            ('bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
+            .format(home=odoo_home)
         )
         logger.debug('Branchesv result:\n%s', cmd_output)
         # Stop instance
@@ -1335,6 +1472,10 @@ class CommandV(Extensible):
         pg_shell = postgresv.PostgresShell(utils.odoo2postgres(db_config))
         pg_shell.terminate_sessions(db_config['db_name'], db_config.get('db_owner'),
                                     db_config.get('db_owner_passwd'))
+        # remove tmp volume
+        tmp_volume = manager.config.get('volumes').get('tmp')
+        logger.info('Removing temp folder volume: %s', tmp_volume)
+        manager.exec_cmd('rm -rf %s' % (tmp_volume))
         # Remove container
         logger.info('Removing container "%s"', container_name)
         manager.remove_container()
@@ -1418,3 +1559,22 @@ class CommandV(Extensible):
             logger.info('Generated the commit history of %s.', name)
         self.sender.send_message(body=res)
         self.done_trigger()
+
+    def new_deactivate(self, config=False):
+        old_db_config = self.instance_manager.db_config.copy()
+        self.instance_manager.db_config.update(config)
+        db_config = self.instance_manager.db_config
+        database = db_config.get('db_name')
+        self.sender.send_message('Deactivating database {}.'.format(database))
+        self.__config.check_config('deactivate')
+        extra_config = {
+            'nginx_url': self.instance_manager.config.get('nginx_url'),
+            'use_mailhog': self.__config.instance_config.get("use_mailhog")}
+        self.instance_manager.deactivate_database(database, extra_config)
+        new_admin = db_config.get('admin', False)
+        if new_admin:
+            db_config.update({'db_name': database})
+            admin_id = self.get_admin_id(db_config=db_config)
+            instancev.InstanceV.change_password(admin_id, new_admin, database, db_config)
+        self.sender.send_message('Database {} deactivated.'.format(database))
+        self.instance_manager.db_config.update(old_db_config)
