@@ -67,6 +67,7 @@ import simplejson as json
 from deployv.helpers import container
 from deployv.helpers.utils import get_error_message, decode
 from deployv.base import errors
+import re
 from retry_decorator import retry
 from time import sleep
 
@@ -487,6 +488,10 @@ class DockerV:
         :type raise_execption: bool
         """
         image_name = image_name or self.config.get('image_name')
+        valid_image = re.match('(?:.*/)?.*/.*:.*', image_name)
+        if not valid_image:
+            logger.warning("The image %s is in bad format and cannot be pulled", image_name)
+            return
         try:
             self.cli.pull(image_name)
         except docker.errors.NotFound:
@@ -624,6 +629,7 @@ class DockerV:
         :raises: :class:`~deployv.base.errors.NoSuchImage` if the image doesn't exist.
         """
         try:
+            self.pull(image_name)
             res = self.cli.inspect_image(image_name)
         except docker.errors.NotFound:
             raise errors.NoSuchImage(image_name)
@@ -692,7 +698,10 @@ class DockerV:
             "hostname": self.config.get("container_hostname"),
             "ports": ports,
             "environment": env_vars,
-            "volumes": volumes
+            "volumes": [
+                volume.split(':')[1] if len(volume.split(':')) > 1 else
+                volume for volume in volumes
+            ]
         }
         if self.config.get("container_name", False):
             container_config.update(
@@ -713,8 +722,15 @@ class DockerV:
             self.__docker_id = self.config.get("container_name").lower()
             self.remove_container()
         hconfig = self.__cli.create_host_config(**host_config)
-        hconfig.get('PortBindings').get("5432/tcp", [{}])[0].\
-            update({'HostIp': '127.0.0.1'})
+        if "5432/tcp" in hconfig.get('PortBindings'):
+            default_network_info = self.__cli.inspect_network('bridge')
+            default_gateway = default_network_info.get('IPAM').get('Config')[0].get('Gateway')
+            host_port = hconfig.get('PortBindings').get("5432/tcp")[0].get('HostPort')
+            hconfig.get('PortBindings').update({
+                "5432/tcp": [
+                    {'HostIp': '127.0.0.1', 'HostPort': host_port},
+                    {'HostIp': default_gateway, 'HostPort': host_port}
+                ]})
         logger.debug('Container config %s', container_config)
         try:
             container_obj = self.__cli.create_container(
