@@ -203,10 +203,10 @@ class PostgresShell:
         if self.__config.get('password', False):
             os.environ['PGPASSWORD'] = self.__config.get('password')
         self.__config.update({'dbname': database})
-        dump_cmd = 'pg_dump {dbname} -O -x -f {0}'
+        dump_cmd = 'pg_dump {dbname} -w -O -x -f {0}'
         if jobs:
             self.__config.update({'num': jobs})
-            dump_cmd = 'pg_dump -j {num} -F d {dbname} -f {0} -x'
+            dump_cmd = 'pg_dump -w  -j {num} -F d {dbname} -f {0} -x'
         if exclude:
             for table in exclude.split(","):
                 dump_cmd += " --exclude-table=%s " % (table)
@@ -299,9 +299,9 @@ class PostgresShell:
         if self.__config.get('password', False):
             os.environ['PGPASSWORD'] = self.__config.get('password')
         self.__config.update({'dbname': database_name})
-        restore_cmd = 'psql {dbname} -f {0}'
+        restore_cmd = 'psql {dbname} -w -f {0}'
         if jobs:
-            restore_cmd = 'pg_restore -j {num} -F d {0} -d {dbname} -x -O -n public'
+            restore_cmd = 'pg_restore -w -j {num} -F d {0} -d {dbname} -x -O -n public'
             cnf.update({'num': str(jobs)})
         restore_cmd = self.__format_cmd(restore_cmd, cnf)
         restore_cmd = restore_cmd.format(dump_name, **cnf)
