@@ -336,7 +336,8 @@ def deactivate(ctx, container, database, admin_password, nginx_url):
 @click.option("-s", "--admin_password", help="Password used to log in as admin in odoo",
               required=True)
 @click.option("-c", "--cformat", default='bz2', required=False,
-              help="Compression format to be used (tar.bz2 or tar.gz)")
+              help=("Compression format to be used: bz2 for tar.bz2, gz for tar.gz,"
+                    " or folder for a backup without compression"))
 @click.option("--upload", default=False, required=False,
               help="URL to upload the deactivated backup")
 @click.option("--remove", is_flag=True, default=False, required=False,
@@ -351,8 +352,11 @@ def deactivate(ctx, container, database, admin_password, nginx_url):
               help=("The numbers of dump in parallel by dumping njobs tables simultaneously."
                     " This option reduces the time of the dump but it also increases"
                     " the load on the database server."))
+@click.option("--migration", is_flag=True, default=False, required=False,
+              help=("If true, the deactivated backup is generated in the format"
+                    " required by odoo's migration scripts"))
 def deactivate_backup(ctx, prefix, store_path, backup_src, admin_password,
-                      cformat, upload, remove, nginx_url, no_fs, jobs):
+                      cformat, upload, remove, nginx_url, no_fs, jobs, migration):
     config_obj = ctx.parent.command.config_parser
     db_config = config_obj.postgres.copy()
     db_config.update({'nginx_url': nginx_url, 'admin': admin_password})
@@ -360,7 +364,7 @@ def deactivate_backup(ctx, prefix, store_path, backup_src, admin_password,
     default_config['group_config'].update({
         'jobs': jobs, 'no_fs': no_fs, 'store_path': store_path,
         'remove': remove, 'deactivating_backup': True, 'backup_src': backup_src,
-        'cformat': cformat, 'prefix': prefix, 'upload': upload})
+        'cformat': cformat, 'prefix': prefix, 'upload': upload, 'migration': migration})
     default_config['instance']['config'].update(db_config)
     config_obj.generate_configuration(default_config)
     command = commandv.CommandV(config_obj)
@@ -378,6 +382,8 @@ def deactivate_backup(ctx, prefix, store_path, backup_src, admin_password,
             process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
             res = process.communicate()[0].strip().decode()
             for backup in res.split("\n"):
+                if not backup:
+                    continue
                 backup_path = os.path.join(config_obj.store_path, backup)
                 utils.clean_files(backup_path)
 
