@@ -21,7 +21,6 @@ from deployv.helpers import container, json_helper
 from deployv.helpers.decompress_helper import DecompressHelper
 from deployv import base
 import deployv_static
-from docker.errors import APIError
 from jinja2 import Environment, FileSystemLoader
 from errno import EEXIST
 from deepdiff import DeepDiff
@@ -638,20 +637,6 @@ def get_backup_src(config_dict=False, deployv_config=False):
     return res
 
 
-def traceback_docker_error(func):
-    """decorator function"""
-    def handle_error(*args, **kwargs):
-        res = {}
-        try:
-            res = func(*args, **kwargs)
-        except (APIError, base.errors.CommandError, base.errors.NoSuchContainer,
-                base.errors.SupervisorStatusError, base.errors.ClientAPIError) as error:
-            logger.error(error)
-            res.update({'error': get_error_message(error)})
-        return res
-    return handle_error
-
-
 def parse_backup_size(size):
     """Parses the backup size provided and returns the value and unit.
 
