@@ -99,6 +99,8 @@ from jinja2 import Template
 
 logger = logging.getLogger(__name__)  # pylint: disable=C0103
 INSTANCE_TYPES = ['test', 'develop', 'updates', 'production']
+ODOO_STAGES = {"production": "production", "updates": "staging",
+               "develop": "dev", "test": "staging"}
 UPDATE = ['develop', 'updates']
 DEACTIVATE = ['test', 'develop', 'updates']
 INSTALL_RIBBON = ['test', 'develop', 'updates']
@@ -121,8 +123,9 @@ class InstanceV(DockerV):
                 env_vars.update({'admin': utils.random_string(6)})
             if not env_vars.get('admin_passwd'):
                 env_vars.update({'admin_passwd': utils.random_string(6)})
-            env_vars.update({'instance_type':
-                             config.instance_config.get('instance_type', 'production'),
+            instance_type = config.instance_config.get('instance_type', 'production')
+            env_vars.update({'instance_type': instance_type,
+                             'odoo_stage': ODOO_STAGES.get(instance_type),
                              'customer': config.instance_config.get('customer_id')})
             self.__full_config.container_config.get('env_vars').update(env_vars)
         super(InstanceV, self).__init__(container_config,
@@ -156,12 +159,15 @@ class InstanceV(DockerV):
             if not postgres_connection.get('success'):
                 return postgres_connection
         self.set_global_env_vars()
-        # Add the PGHOSt env var here because we only know which value to set after
+        # Add the PGHOST and PGPORT env vars here because we only know which values to set after
         # the container is created, mainly because we don't know if the postgres is in
         # another container.
         self.exec_cmd(
             "/bin/bash -c 'echo export PGHOST={db_host} | tee -a /home/odoo/.profile'"
             .format(db_host=self.config.get('env_vars').get('db_host')))
+        self.exec_cmd(
+            "/bin/bash -c 'echo export PGPORT={db_port} | tee -a /home/odoo/.profile'"
+            .format(db_port=self.config.get('env_vars').get('db_port')))
         logger.info('Odoo container "%s" created. Ports %s',
                     info.get('name'), info.get('ports'))
         return info
@@ -663,26 +669,23 @@ class InstanceV(DockerV):
 
         :return: A dict with the basic info of container
         """
-        if not self.__full_config.container_config.get('postgres_container'):
+        if (not self.__full_config.container_config.get('postgres_container') or
+                self.__full_config.backup_type in ['btrfs', 'zfs']):
             return {}
         destination = '/var/log/pg_log'
         if self.config.get('keep_db') or self.__full_config.reloading_config:
             info = self.inspect(self.config.get('postgres_container_name'))
-            db_host = info.get('NetworkSettings').get('IPAddress')
+            db_host = info.get('NetworkSettings').get('Gateway')
             self.config.update({'pg_log_postgres': self.get_specify_volumen(info, destination)})
-            self.__full_config.instance_config.get("config").update({'db_host': db_host})
+            db_port = self.get_db_port(info)
+            self.__full_config.instance_config.get("config").update({
+                'db_host': db_host, 'db_port': db_port})
             self.config.get('env_vars').update({
-                'db_port': 5432,
-                'db_host': db_host, 'external_db_port': self.get_db_port(info)})
+                'db_port': db_port,
+                'db_host': db_host, 'external_db_port': info})
             return {}
-        try:
-            image_data = self.inspect_image(self.config.get('image_name'))
-            vars_list = image_data.get('Config').get('Env')
-        except errors.NoSuchImage:
-            vars_list = []
         ins_config = self.config.copy()
-        image_vars = container.parse_env_vars(vars_list)
-        psql_version = image_vars.get('psql_version', '9.6')
+        psql_version = self.get_postgres_version()
         psql_image = "postgres:{version}".format(version=psql_version)
         role = 'createdb'
         if self.config.get('full_stack'):
@@ -707,20 +710,41 @@ class InstanceV(DockerV):
             format(user=self.db_config.get('db_user'), role=role,
                    password=self.db_config.get('db_password'))
         self.exec_cmd(cmd, user="postgres")
+        # Create onwer user in postgreSQL
+        cmd = "psql -c \"create user {user} with password '{password}' superuser\"".\
+            format(user=self.db_config.get('db_owner'),
+                   password=self.db_config.get('db_owner_passwd'))
+        self.exec_cmd(cmd, user="postgres")
         info = self.inspect()
         self.config.update(ins_config)
         self.config.get('volumes').pop('pg_log', False)
         self.config.update({
             'pg_log_postgres': self.get_specify_volumen(info, destination)})
-        db_host = info.get('NetworkSettings').get('IPAddress')
+        db_host = info.get('NetworkSettings').get('Gateway')
+        db_port = self.get_db_port(info)
         self.config.get('env_vars').update({
-            'db_host': db_host, 'external_db_port': self.get_db_port(info)})
-        self.__full_config.instance_config.get('config').update({'db_host': db_host})
+            'db_host': db_host, 'external_db_port': db_port, 'db_port': db_port})
+        self.__full_config.instance_config.get('config').update({
+            'db_host': db_host, 'db_port': db_port})
         # Don't expose the 5432 port in the odoo container
         # if a postgres container was created
         self.config.get('ports').pop('5432', False)
         return res
 
+    def get_postgres_version(self):
+        """Helper method used to obtain the postgres version required for this version of odoo.
+        This makes it easier to get the information used for the btrfs/zfs requests and the
+        information required to create the postgres containers.
+        """
+        try:
+            image_data = self.inspect_image(self.config.get('image_name'))
+            vars_list = image_data.get('Config').get('Env')
+        except errors.NoSuchImage:
+            vars_list = []
+        image_vars = container.parse_env_vars(vars_list)
+        psql_version = image_vars.get('psql_version', '9.6')
+        return psql_version
+
     def get_specify_volumen(self, info, volume_path):
         """Get a specify volumen from the information of container.
         """
@@ -799,7 +823,6 @@ class InstanceV(DockerV):
         """
         files = []
         odoo_home = self.config.get('env_vars').get('odoo_home')
-        odoo_user = self.config.get('env_vars').get('odoo_user')
         jobs = self.__full_config.jobs
         postgres_cfg = self.db_config
         data_dir = data_dir or postgres_cfg.get('data_dir')
@@ -845,8 +868,8 @@ class InstanceV(DockerV):
             logger.info('There is not attachments folder to backup')
         try:
             cmd_output = self.exec_cmd(
-                ('su {user} -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
-                .format(user=odoo_user, home=odoo_home)
+                ('bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
+                .format(home=odoo_home)
             )
             logger.debug('Branchesv result:\n%s', cmd_output)
             json_repos = path.join(self.temp_folder, 'repositories.json')
@@ -996,8 +1019,11 @@ class InstanceV(DockerV):
         logger.debug('Creating auxiliary container')
         cmd = ' && '.join(commands)
         cmd = cmd.format(fsname=fs_name)
-        container_cfg = self.inspect()
-        image = container_cfg.get('Config').get('Image') or self.config.get('image_name')
+        try:
+            container_cfg = self.inspect()
+        except errors.NoSuchContainer:
+            container_cfg = {}
+        image = container_cfg.get('Config', {}).get('Image') or self.config.get('image_name')
         self.exec_cmd_image("bash -c '%s'" % (cmd), volumes, image=image)
 
     def execute_query(self, db_config, query, allow_errors=False):
@@ -1053,28 +1079,29 @@ class InstanceV(DockerV):
         :return: a dict with the added, deleted and updated views. In the case of updated will
             return the diff between the production database and the updates database
         """
-        res = {
-            'updated': list(),
-            'added': list(),
-            'deleted': list()
-        }
-        checked = []
+        res = {'updated': [], 'added': [], 'deleted': []}
+        copy_orig_views = original_views[:]  # to maintain original_views unchanged
+        total = len(copy_orig_views)
         for view_modified in modified_views:
-            for view_original in original_views:
-                if view_original['xml_id'] == view_modified['xml_id']:
-                    checked.append(view_original)
-                    if view_original['arch'] != view_modified['arch']:
-                        res.get('updated').append({
-                            'xml_id': view_original['xml_id'],
-                            'original': view_original['arch'],
+            found = False
+            ind = 0
+            while not found and ind < total:
+                if copy_orig_views[ind]['xml_id'] == view_modified['xml_id']:
+                    if copy_orig_views[ind]['arch'] != view_modified['arch']:
+                        res['updated'].append({
+                            'xml_id': copy_orig_views[ind]['xml_id'],
+                            'original': copy_orig_views[ind]['arch'],
                             'modified': view_modified['arch']
                         })
-                    break
-            else:
-                res.get('added').append(view_modified)
-        for original_view in original_views:
-            if original_view not in checked:
-                res.get('deleted').append(original_view)
+                    found = True
+                    copy_orig_views.pop(ind)
+                    total -= 1
+                else:
+                    ind += 1   # only in else, when no pop is done in copy_orig_views
+            if not found:
+                res['added'].append(view_modified)
+        for original_view in copy_orig_views:
+            res['deleted'].append(original_view)
         return res
 
     def get_translations(self, database):
@@ -1098,37 +1125,37 @@ class InstanceV(DockerV):
         :return: A dict with the added, updated and removed translations between the production
                  database and the updates database.
         """
-        checked = list()
-        res = {
-            'updated': list(),
-            'added': list(),
-            'deleted': list()
-        }
+        res = {'updated': [], 'added': [], 'deleted': []}
+        copy_orig_translations = original_translations[:]  # to maintain original_translations
+        total = len(copy_orig_translations)
         for modified_translation in modified_translations:
-            for original_translation in original_translations:
-                if original_translation['id'] == modified_translation['id']:
-                    checked.append(original_translation)
-                    if original_translation['value'] != modified_translation['value']:
-                        res.get('updated').append({
-                            'name': original_translation['name'],
-                            'module': original_translation['module'],
-                            'original': original_translation['value'],
+            found = False
+            ind = 0
+            while not found and ind < total:
+                if copy_orig_translations[ind]['id'] == modified_translation['id']:
+                    if copy_orig_translations[ind]['value'] != modified_translation['value']:
+                        res['updated'].append({
+                            'name': copy_orig_translations[ind]['name'],
+                            'module': copy_orig_translations[ind]['module'],
+                            'original': copy_orig_translations[ind]['value'],
                             'modified': modified_translation['value']
                         })
-                    break
-            else:
-                res.get('added').append({
+                    found = True
+                    copy_orig_translations.pop(ind)
+                    total -= 1
+                ind += 1
+            if not found:
+                res['added'].append({
                     'name': modified_translation['name'],
                     'module': modified_translation['module'],
                     'value': modified_translation['value']
                 })
-        for original_translation in original_translations:
-            if original_translation not in checked:
-                res.get('deleted').append({
-                    'name': original_translation['name'],
-                    'module': original_translation['module'],
-                    'value': original_translation['value'],
-                })
+        for original_translation in copy_orig_translations:
+            res['deleted'].append({
+                'name': original_translation['name'],
+                'module': original_translation['module'],
+                'value': original_translation['value'],
+            })
         return res
 
     def get_menus(self, database):
@@ -1174,32 +1201,34 @@ class InstanceV(DockerV):
         return res
 
     def compare_menus(self, original_menus, modified_menus):
-        res = {
-            'updated': list(),
-            'added': list(),
-            'deleted': list()
-        }
-        for uxml_id, urecord in modified_menus.items():
-            if uxml_id in original_menus \
-                    and original_menus[uxml_id]['name'] != urecord['name']:
-                res['updated'].append({
-                    'xml_id': uxml_id,
-                    'original': original_menus[uxml_id]['name'],
-                    'modified': urecord['name'],
-                    'hierarchypath': urecord.get('hierarchypath')
-                })
-            if uxml_id not in original_menus:
+        """Compare all the menu items from two json values, obtained from backup database,
+        and return a proper report
+
+        :param original_menus: A list of dicts, containing info of original menu items.
+        :param modified_menus: A list of dicts, containing info of updated menu items.
+        :return: A dict with the added, updated and removed menus between both json values."""
+        res = {'updated': [], 'added': [], 'deleted': []}
+        for uxml_id in modified_menus:
+            if uxml_id in original_menus:
+                if original_menus[uxml_id]['name'] != modified_menus[uxml_id]['name']:
+                    res['updated'].append({
+                        'xml_id': uxml_id,
+                        'original': original_menus[uxml_id]['name'],
+                        'modified': modified_menus[uxml_id]['name'],
+                        'hierarchypath': modified_menus[uxml_id].get('hierarchypath')
+                    })
+            else:
                 res['added'].append({
                     'xml_id': uxml_id,
-                    'name': urecord['name'],
-                    'hierarchypath': urecord.get('hierarchypath')
+                    'name': modified_menus[uxml_id]['name'],
+                    'hierarchypath': modified_menus[uxml_id].get('hierarchypath')
                 })
-        for pxml_id, precord in original_menus.items():
+        for pxml_id in original_menus.keys():
             if pxml_id not in modified_menus:
                 res['deleted'].append({
-                    'xml_id': precord['xml_id'],
-                    'name': precord['name'],
-                    'hierarchypath': precord.get('hierarchypath')
+                    'xml_id': original_menus[pxml_id]['xml_id'],
+                    'name': original_menus[pxml_id]['name'],
+                    'hierarchypath': original_menus[pxml_id].get('hierarchypath')
                 })
         return res
 
@@ -1232,32 +1261,39 @@ class InstanceV(DockerV):
         return res
 
     def compare_fields(self, original_fields, modified_fields):
-        res = {
-            'updated': list(), 'added': list(), 'deleted': list(),
-        }
-        checked = []
+        """Compare all the field items from two json values, obtained from backup database,
+        and return a proper report
+
+        :param original_fields: A list of dicts, containing info of original fields items.
+        :param modified_fields: A list of dicts, containing info of updated fields items.
+        :return: A dict with the added, updated and removed fields between both json values."""
+        res = {'updated': [], 'added': [], 'deleted': []}
+        copy_orig_fields = original_fields[:]  # to maintain original_fields unchanged
+        total = len(copy_orig_fields)
         for modified in modified_fields:
-            for original in original_fields:
-                if modified['model'] == original['model']\
-                        and modified['name'] == original['name']:
-                    checked.append(original)
-                    if modified['type'] != original['type']\
-                            or modified['description'] != original['description']:
-                        updated = {
-                            'model': original['model'],
-                            'name': original['name'],
-                            'original': {'type': original['type'],
-                                         'description': original['description']},
+            found = False
+            ind = 0
+            while not found and ind < total:
+                if modified['model'] == copy_orig_fields[ind]['model'] \
+                        and modified['name'] == copy_orig_fields[ind]['name']:
+                    if modified['type'] != copy_orig_fields[ind]['type'] \
+                            or modified['description'] != copy_orig_fields[ind]['description']:
+                        res['updated'].append({
+                            'model': copy_orig_fields[ind]['model'],
+                            'name': copy_orig_fields[ind]['name'],
+                            'original': {'type': copy_orig_fields[ind]['type'],
+                                         'description': copy_orig_fields[ind]['description']},
                             'modified': {'type': modified['type'],
                                          'description': modified['description']},
-                        }
-                        res.get('updated').append(updated)
-                    break
-            else:
-                res.get('added').append(modified)
-        for original in original_fields:
-            if original not in checked:
-                res.get('deleted').append(original)
+                        })
+                    found = True
+                    copy_orig_fields.pop(ind)
+                    total -= 1
+                ind += 1
+            if not found:
+                res['added'].append(modified)
+        for original_field in copy_orig_fields:
+            res['deleted'].append(original_field)
         return res
 
     def get_odoo_users(self, database, user_id=False):
@@ -1575,7 +1611,9 @@ class InstanceV(DockerV):
                         "= 'value' and model = 'ir.config_parameter';")
         config_parameter_id = self.execute_query(db_config, query_config)
         config_parameter_id = config_parameter_id and config_parameter_id[0].get('id')
-        extra_config.update({'field_id': config_parameter_id, 'uuid': uuid4()})
+        extra_config.update({
+            'field_id': config_parameter_id, 'uuid': uuid4(),
+            'migration': self.__full_config.migration})
         # Sqls to be executed
         template_path = deployv_static.get_template_path('deactivation.jinja')
         with open(template_path, 'r') as obj:
@@ -1604,6 +1642,8 @@ class InstanceV(DockerV):
         """
 
         env_vars = self.docker_env or {}
+        if self.__full_config.backup_type in ['btrfs', 'zfs']:
+            env_vars = env_vars or self.config.get('env_vars')
         res = self.__full_config.instance_config.get("config").copy()
         logger.debug(json.dumps(env_vars, sort_keys=True, indent=4))
         logger.debug("Instance config dict from json: %s", str(res))
