@@ -9,7 +9,7 @@ from deployv.messaging.basemsg import BasicMessage
 _logger = logging.getLogger('deployv')
 
 
-class HttpSenderV(object):
+class HttpSenderV:
     """ A simple rabbit mq interface just pass the config parameters
     and send the message
 
